﻿using UnityEngine;
using System.Collections.Generic;

public class Character : MonoBehaviour {

    enum DamageType { PHYSICAL, FIRE, COLD }

	// Use this for initialization
	void OnEnable () {
        InventoryItems = new List<Item>();
        OngoingEffects = new List<EffectData>();
	}

    public float HitPoints { get; protected set; }

    public float MovementSpeed { get; set; }

    protected List<Item> InventoryItems;

    protected List<EffectData> OngoingEffects;

    public delegate void DeathEventHandler(Character c);
    public event DeathEventHandler OnDeath;

	// Update is called once per frame
	void Update () {
	    // Move by MovementSpeed

        if( Input.GetButtonDown("Consume Inventory 1 Button") ) {
            // Do we HAVE something in the first slot?
            if(InventoryItems[0] != null)
            {
                // Use it!
                InventoryItems[0].Consume( this );
            }

        }

        // Now call the OnUpdate event on OngoingEffects
        List<EffectData> copyOfOngoing = new List<EffectData>(OngoingEffects);
        foreach(EffectData effect in copyOfOngoing)
        {
            effect.OnUpdate(this, Time.deltaTime);
        }
	}

    public void PickupItem( Item item )
    {
        InventoryItems.Add(item);
        if(item.EquippedEffectData != null && item.EquippedEffectData.OnBeginEffect != null)
        {
            item.EquippedEffectData.OnBeginEffect( this );
        }
    }

    public void DropItem( Item item )
    {
        InventoryItems.Remove(item);
        if(item.EquippedEffectData != null && item.EquippedEffectData.OnEndEffect != null)
        {
            item.EquippedEffectData.OnEndEffect( this );
        }

    }

    public void AddOngoingEffect(EffectData effect)
    {
        OngoingEffects.Add(effect);
        effect.OnBeginEffect(this);
    }

    public void RemoveOngoingEffect(EffectData effect)
    {
        OngoingEffects.Remove(effect);
        effect.OnEndEffect(this);
    }

    public void HealDamage( float amt )
    {
        HitPoints += amt;
    }

    public void TakeDamage( DamageData dd )
    {
        HitPoints -= dd.Amount;

        // Do we have a "Thorns" type effect, which means we have to hurt the person
        // who injured us


        if(HitPoints <= 0)
        {
            // Die();
            // Who killed us?  Because we need to award them points
        }
    }


}
