﻿using UnityEngine;
using System.Collections;

public class GameManager : MonoBehaviour {

	// Use this for initialization
	void Start () {

        Character player1 = new Character();

        Item healthPot = new Item();
        healthPot.OnConsume += GameManager.OnConsume_HealthPotion;
        player1.PickupItem(healthPot);

        Item healthPot2 = new Item();
        healthPot2.OnConsume += GameManager.OnConsume_BigHealthPotion;
        player1.PickupItem(healthPot2);

        Item bootsOfSpeed = new Item();
        bootsOfSpeed.EquippedEffectData = new EffectData();
        bootsOfSpeed.EquippedEffectData.OnBeginEffect += OnEquip_BootsOfSpeed;
        bootsOfSpeed.EquippedEffectData.OnEndEffect   += OnUnEquip_BootsOfSpeed;
        player1.PickupItem(bootsOfSpeed);

        Item healOverTimePotion= new Item();
        healOverTimePotion.OnConsume += (item, character) => { 
            OnConsume_HealOverTimePotion(item, character, 9, 10); 
        };
        player1.PickupItem(healOverTimePotion);

        Item speedPotion= new Item();
        speedPotion.OnConsume += OnConsume_SpeedPotion;
        player1.PickupItem(speedPotion);

        Item someItemWithCooldown = new Item();
        someItemWithCooldown.EquippedEffectData = new EffectData();
        // If the item has passive stats, fill out EquippedEffectData

        // We can define an ADDITIONAL OnUpdate for the effect data to
        // update the cooldown

        someItemWithCooldown.EquippedEffectData.OnUpdate += (character, deltaTime) => {
            someItemWithCooldown.cooldownRemaining -= deltaTime;
        };


        someItemWithCooldown.OnConsume += OnConsume_SomeItemWithCooldown;
        someItemWithCooldown.cooldownWhenUsed = 60;
        player1.PickupItem(someItemWithCooldown);
    }
	
	// Update is called once per frame
	void Update () {
	
	}


    /////////////////
    /// The code below could really be ANYWHERE in your program
    /// And probably will be a "static" function -- not linked to
    /// a specific instance of an object
    /// 

    public static void OnConsume_HealthPotion( Item item, Character consumer )
    {
        consumer.HealDamage( 10 );

        // This is a one-shot item, so lets remove ourselves from
        // the player's inventory
        consumer.DropItem(item);
    }


    public static void OnConsume_BigHealthPotion( Item item, Character consumer )
    {
        consumer.HealDamage( 50 );
        consumer.DropItem(item);
    }

    public static void OnConsume_HealOverTimePotion( Item item, Character consumer, float totalDuration, float amountPerSecond )
    {
        EffectData healOverTimeEffect = new EffectData();

        float duration = totalDuration;

        healOverTimeEffect.OnUpdate      += (character, deltaTime) => {

            // HoT potion ticks every second
            int numTicksLeft = Mathf.FloorToInt(duration);

            duration -= deltaTime;


            if( Mathf.FloorToInt(duration) < numTicksLeft )
            {
                // Do a heal tick!
                character.HealDamage(amountPerSecond);
            }


            if(duration <= 0)
            {
                // Speed potion has expired
                character.RemoveOngoingEffect(healOverTimeEffect);
            }
        };


        consumer.AddOngoingEffect(healOverTimeEffect);

        consumer.DropItem(item);
    }

    public static void OnConsume_SpeedPotion( Item item, Character consumer )
    {
        EffectData speedEffect = new EffectData();

        speedEffect.OnBeginEffect += (c) => { c.MovementSpeed += 10; };
        speedEffect.OnEndEffect   += (c) => { c.MovementSpeed -= 10; };

        float duration = 60;

        speedEffect.OnUpdate      += (character, deltaTime) => {
            duration -= deltaTime;
            if(duration <= 0)
            {
                // Speed potion has expired
                character.RemoveOngoingEffect(speedEffect);
            }
        };

        consumer.AddOngoingEffect(speedEffect);

        consumer.DropItem(item);

    }

    public static void OnEquip_BootsOfSpeed( Character c )
    {
        c.MovementSpeed += 10;
    }

    public static void OnUnEquip_BootsOfSpeed( Character c )
    {
        c.MovementSpeed -= 10;
    }

    public static void OnConsume_SomeItemWithCooldown( Item item, Character character)
    {
        if(item.cooldownRemaining <= 0)
        {
            // Do the "thing" the item is supposed to do

            // Reset the cooldown
            item.cooldownRemaining = item.cooldownWhenUsed;
        }

        // NOTE: We don't "drop" this item after consuming
    }
}
